# Copyright 2019 Red Hat
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import

from oslo_config import cfg


def register_tobiko_options(conf):
    conf.register_opts(
        group=cfg.OptGroup('nova'),
        opts=[cfg.StrOpt('image',
                         help="Default image for new server instances"),
              cfg.StrOpt('flavor',
                         help="Default flavor for new server instances"),
              cfg.StrOpt('key_file', default='~/.ssh/id_rsa',
                         help="Default SSH key to login to server instances"),
              cfg.StrOpt('username', default='cirros',
                         help="Default username to login to server instances"),
              cfg.StrOpt('password', default='gocubsgo',
                         help="Default password to login to server instances"),
              ])
